==> rx-0.11.1-sources.jar: app/raybritton/tokenstorage/RxTokenStorage.kt <==
package app.raybritton.tokenstorage

import app.raybritton.tokenstorage.crypto.Crypto
import app.raybritton.tokenstorage.keyCrypto.KeyCrypto
import app.raybritton.tokenstorage.keyCrypto.NoKeyCrypto
import app.raybritton.tokenstorage.persistence.Persistence
import io.reactivex.BackpressureStrategy
import io.reactivex.Completable
import io.reactivex.Flowable
import io.reactivex.subjects.ReplaySubject

class RxTokenStorage(crypto: Crypto,
                     persistence: Persistence,
                     keyCrypto: KeyCrypto = NoKeyCrypto()) {
    private val storage = TokenStorage(crypto, persistence, keyCrypto)

    private val loadSubjects = mutableMapOf<String, ReplaySubject<Optional<String>>>()
    private val containSubjects = mutableMapOf<String, ReplaySubject<Boolean>>()

    private val keysSubject = ReplaySubject.create<List<String>>(1)

    fun clearAll(): Completable {
        return Completable.fromCallable {
            storage.clearAll()
            loadSubjects.values.forEach {
                it.onNext(Optional.None)
            }
            containSubjects.values.forEach {
                it.onNext(false)
            }
            keysSubject.onNext(listOf())
        }
    }

    fun clear(key: String): Completable {
        return Completable.fromCallable {
            storage.clear(key)
            reload(key)
        }
    }

    fun save(key: String, plaintext: String): Completable {
        return Completable.fromCallable {
            storage.save(key, plaintext)
            reload(key)
        }
    }

    fun load(key: String): Flowable<Optional<String>> {
        storage.crypto.verify()
        storage.keyCrypto.verify()
        if (loadSubjects.containsKey(key)) {
            return loadSubjects[key]!!.share().toFlowable(BackpressureStrategy.LATEST)
        } else {
            val subject = ReplaySubject.createWithSize<Optional<String>>(1)
            loadSubjects[key] = subject
            subject.onNext(Optional.auto(loadString(key)))
            return subject.share().toFlowable(BackpressureStrategy.LATEST)
        }
    }

    fun keys(): Flowable<List<String>> {
        storage.crypto.verify()
        storage.keyCrypto.verify()
        return keysSubject.share().toFlowable(BackpressureStrategy.LATEST)
                .startWith(storage.keys())
    }

    fun contains(key: String): Flowable<Boolean> {
        storage.crypto.verify()
        storage.keyCrypto.verify()
        if (containSubjects.containsKey(key)) {
            return containSubjects[key]!!.share().toFlowable(BackpressureStrategy.LATEST)
        } else {
            val subject = ReplaySubject.createWithSize<Boolean>(1)
            containSubjects[key] = subject
            subject.onNext(loadContains(key))
            return subject.share().toFlowable(BackpressureStrategy.LATEST)
        }
    }

    fun clearAllSync() {
        storage.clearAll()
        loadSubjects.values.forEach {
            it.onNext(Optional.None)
        }
        containSubjects.values.forEach {
            it.onNext(false)
        }
        keysSubject.onNext(listOf())
    }

    fun clearSync(key: String) {
        storage.clear(key)
        reload(key)
    }

    fun saveSync(key: String, plaintext: String) {
        storage.save(key, plaintext)
        reload(key)
    }

    fun loadSync(key: String): String? {
        return storage.load(key)
    }

    fun containsSync(key: String): Boolean {
        return storage.contains(key)
    }

    fun wrap(key: String): RxTokenWrapper {
        return RxTokenWrapper(this, key)
    }

    private fun loadContains(key: String): Boolean {
        return storage.contains(key)
    }

    private fun loadString(key: String): String? {
        return storage.load(key)
    }
    private fun reload(key: String) {
        loadSubjects[key]?.onNext(Optional.auto(loadString(key)))
        containSubjects[key]?.onNext(loadContains(key))
        keysSubject.onNext(storage.keys())
    }

}

==> rx-0.11.4-sources.jar: app/raybritton/tokenstorage/RxTokenStorage.kt <==
package app.raybritton.tokenstorage

import app.raybritton.tokenstorage.crypto.Crypto
import app.raybritton.tokenstorage.keyCrypto.KeyCrypto
import app.raybritton.tokenstorage.persistence.Persistence
import io.reactivex.BackpressureStrategy
import io.reactivex.Completable
import io.reactivex.Flowable
import io.reactivex.subjects.ReplaySubject

class RxTokenStorage<C : Crypto, P : Persistence, KC : KeyCrypto>(
        crypto: C,
        persistence: P,
        keyCrypto: KC) {
    private val storage = TokenStorage(crypto, persistence, keyCrypto)

    private val loadSubjects = mutableMapOf<String, ReplaySubject<Optional<String>>>()
    private val containSubjects = mutableMapOf<String, ReplaySubject<Boolean>>()

    private val keysSubject = ReplaySubject.create<List<String>>(1)

    fun clearAll(): Completable {
        return Completable.fromCallable {
            storage.clearAll()
            loadSubjects.values.forEach {
                it.onNext(Optional.None)
            }
            containSubjects.values.forEach {
                it.onNext(false)
            }
            keysSubject.onNext(listOf())
        }
    }

    fun clear(key: String): Completable {
        return Completable.fromCallable {
            storage.clear(key)
            reload(key)
        }
    }

    fun save(key: String, plaintext: String): Completable {
        return Completable.fromCallable {
            storage.save(key, plaintext)
            reload(key)
        }
    }

    fun load(key: String): Flowable<Optional<String>> {
        storage.crypto.verify()
        storage.keyCrypto.verify()
        if (loadSubjects.containsKey(key)) {
            return loadSubjects[key]!!.share().toFlowable(BackpressureStrategy.LATEST)
        } else {
            val subject = ReplaySubject.createWithSize<Optional<String>>(1)
            loadSubjects[key] = subject
            subject.onNext(Optional.auto(loadString(key)))
            return subject.share().toFlowable(BackpressureStrategy.LATEST)
        }
    }

    fun keys(): Flowable<List<String>> {
        storage.crypto.verify()
        storage.keyCrypto.verify()
        return keysSubject.share().toFlowable(BackpressureStrategy.LATEST)
                .startWith(storage.keys())
    }

    fun contains(key: String): Flowable<Boolean> {
        storage.crypto.verify()
        storage.keyCrypto.verify()
        if (containSubjects.containsKey(key)) {
            return containSubjects[key]!!.share().toFlowable(BackpressureStrategy.LATEST)
        } else {
            val subject = ReplaySubject.createWithSize<Boolean>(1)
            containSubjects[key] = subject
            subject.onNext(loadContains(key))
            return subject.share().toFlowable(BackpressureStrategy.LATEST)
        }
    }

    fun clearAllSync() {
        storage.clearAll()
        loadSubjects.values.forEach {
            it.onNext(Optional.None)
        }
        containSubjects.values.forEach {
            it.onNext(false)
        }
        keysSubject.onNext(listOf())
    }

    fun clearSync(key: String) {
        storage.clear(key)
        reload(key)
    }

    fun saveSync(key: String, plaintext: String) {
        storage.save(key, plaintext)
        reload(key)
    }

    fun loadSync(key: String): String? {
        return storage.load(key)
    }

    fun containsSync(key: String): Boolean {
        return storage.contains(key)
    }

    fun wrap(key: String): RxTokenWrapper {
        return RxTokenWrapper(this, key)
    }

    private fun loadContains(key: String): Boolean {
        return storage.contains(key)
    }

    private fun loadString(key: String): String? {
        return storage.load(key)
    }

    private fun reload(key: String) {
        loadSubjects[key]?.onNext(Optional.auto(loadString(key)))
        containSubjects[key]?.onNext(loadContains(key))
        keysSubject.onNext(storage.keys())
    }

}
